package com.wzy.es.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.wzy.common.domain.Product;
import com.wzy.common.domain.request.Req;
import com.wzy.common.result.PageResult;
import com.wzy.common.result.Result;
import com.wzy.common.utils.StringUtils;
import com.wzy.es.service.EsService;
import org.elasticsearch.action.search.SearchRequest;
import org.elasticsearch.action.search.SearchResponse;
import org.elasticsearch.client.RequestOptions;
import org.elasticsearch.client.RestHighLevelClient;
import org.elasticsearch.common.text.Text;
import org.elasticsearch.index.query.BoolQueryBuilder;
import org.elasticsearch.index.query.QueryBuilders;
import org.elasticsearch.search.SearchHit;
import org.elasticsearch.search.SearchHits;
import org.elasticsearch.search.builder.SearchSourceBuilder;
import org.elasticsearch.search.fetch.subphase.highlight.HighlightBuilder;
import org.elasticsearch.search.fetch.subphase.highlight.HighlightField;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Map;

@Service
public class EsServiceImpl implements EsService {

    @Autowired
    private RestHighLevelClient restHighLevelClient;

    private static final String INDEX_NAME = "product";

    @Override
    public Result list(Req req) {
        long total = 0;
        ArrayList<Product> list = new ArrayList<>();

        try {
            SearchRequest searchRequest = new SearchRequest(INDEX_NAME);
            SearchSourceBuilder searchSourceBuilder = new SearchSourceBuilder();
            BoolQueryBuilder boolQueryBuilder = QueryBuilders.boolQuery();
            if (StringUtils.isNotEmpty(req.getProductName())) {
                boolQueryBuilder.must(QueryBuilders.matchQuery("productName",req.getProductName()));
            }
            if (StringUtils.isNotEmpty(req.getProductType())) {
                boolQueryBuilder.must(QueryBuilders.matchQuery("productType",req.getProductType()));
            }
            searchSourceBuilder.query(boolQueryBuilder);
            HighlightBuilder highlightBuilder = new HighlightBuilder();
            highlightBuilder.field("goodsName").preTags("<font colour=red>").postTags("</font>");
            searchSourceBuilder.highlighter(highlightBuilder);
            searchRequest.source(searchSourceBuilder);
            SearchResponse search = restHighLevelClient.search(searchRequest, RequestOptions.DEFAULT);
            SearchHits hits = search.getHits();
            total = hits.getTotalHits().value;
            for (SearchHit hit : hits.getHits()) {
                String sourceAsString = hit.getSourceAsString();
                Product product = JSONObject.parseObject(sourceAsString, Product.class);
                product.setProductId(hit.getId());
                Map<String, HighlightField> highlightFields = hit.getHighlightFields();
                if (null != highlightFields) {
                    HighlightField highlightField = highlightFields.get("productName");
                    if (null != highlightField) {
                        String str = "";
                        for (Text fragment : highlightField.fragments()) {
                            str += fragment;
                        }
                        product.setProductName(str);
                    }
                }
                list.add(product);
            }

        } catch (IOException e) {
            throw new RuntimeException(e);
        }
        return PageResult.toResult(total,list);
    }
}
