package com.wzy.es.sync;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.serializer.SimplePropertyPreFilter;
import com.wzy.common.domain.Product;
import com.wzy.common.result.Result;
import com.wzy.es.feign.ProductFeignService;
import lombok.extern.log4j.Log4j2;
import org.elasticsearch.action.admin.indices.cache.clear.ClearIndicesCacheRequest;
import org.elasticsearch.action.admin.indices.cache.clear.ClearIndicesCacheResponse;
import org.elasticsearch.action.bulk.BulkRequest;
import org.elasticsearch.action.index.IndexRequest;
import org.elasticsearch.client.RequestOptions;
import org.elasticsearch.client.RestHighLevelClient;
import org.elasticsearch.common.xcontent.XContentType;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

@Component
@Log4j2
public class EsSync {

    @Autowired
    private ProductFeignService productFeignService;

    @Autowired
    private RestHighLevelClient restHighLevelClient;

    private static final String INDEX_NAME = "product";

    @Scheduled(cron = "0 * * * * *")
    public void runEs() throws IOException {
        Result<List<Product>> productList = productFeignService.findProductList();
        List<Product> data = productList.getData();
        long l = System.currentTimeMillis();
        if (!CollectionUtils.isEmpty(data)) {
            BulkRequest bulkRequest = new BulkRequest();
            data.forEach(item -> {
                SimplePropertyPreFilter filter = new SimplePropertyPreFilter();
                filter.getExcludes().addAll(new ArrayList<String>(){{
                    add("productId");
                }});
                bulkRequest.add(
                        new IndexRequest(INDEX_NAME)
                                .source(JSON.toJSONString(item,filter), XContentType.JSON)
                                .id(item.getProductId())
                );
            });
            restHighLevelClient.bulk(bulkRequest, RequestOptions.DEFAULT);
            log.info("功能:同步ES消耗:{}毫秒",(System.currentTimeMillis())-l);

            ClearIndicesCacheRequest cacheRequest = new ClearIndicesCacheRequest();
            cacheRequest.indices(INDEX_NAME);
            cacheRequest.fieldDataCache(true);
            cacheRequest.queryCache(true);
            ClearIndicesCacheResponse cacheResponse = restHighLevelClient.indices().clearCache(cacheRequest, RequestOptions.DEFAULT);
            log.info("清除ES索引缓存成功"+cacheResponse);
            Date date = new Date();
            log.info("当前时间:"+date);
        }
    }
}
