package com.wzy.product.controller;

import com.alibaba.fastjson.JSONObject;
import com.wzy.common.domain.Check;
import com.wzy.common.domain.Product;
import com.wzy.common.domain.Rule;
import com.wzy.common.domain.User;
import com.wzy.common.result.Result;
import com.wzy.product.service.ProductService;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import java.util.List;

@RestController
@Log4j2
public class ProductController {

    @Autowired
    private ProductService productService;

    @Autowired
    private HttpServletRequest request;

    @GetMapping("/findProductList")
    public Result<List<Product>> findProductList(){
        List<Product> findProductList = productService.findProductList();
        return Result.success(findProductList);
    }

    @GetMapping("/ruleList")
    @Scheduled(cron = "0 * * * * *")
    public Result<List<Rule>> ruleList(){
        List<Rule> ruleList = productService.ruleList();
        return Result.success(ruleList);
    }

    @PostMapping("/ruleSave")
    public Result<String> ruleSave(@RequestBody Rule rule){
        log.info("功能:新增规则,请求URI:{},请求方式:{},请求参数:{}",request.getRequestURI(),
                request.getMethod(),rule);
        Result result = productService.ruleSave(rule);
        log.info("功能:新增规则,请求URI:{},请求方式:{},响应结果:{}",request.getRequestURI(),
                request.getMethod(), JSONObject.toJSONString(result));
        return result;
    }

    @GetMapping("/choiceProduct")
    public Result<List<Product>> choiceProduct(){
        List<Product> choiceProduct = productService.choiceProduct();
        return Result.success(choiceProduct);
    }

    @PostMapping("/productSave")
    public Result<String> productSave(@RequestBody Product product){
        log.info("功能:新增商品,请求URI:{},请求方式:{},请求参数:{}",request.getRequestURI(),
                request.getMethod(),product);
        Result result = productService.productSave(product);
        log.info("功能:新增商品,请求URI:{},请求方式:{},响应结果:{}",request.getRequestURI(),
                request.getMethod(), JSONObject.toJSONString(result));
        return result;
    }

    @PostMapping("/startUp/{productId}")
    public Result<String> startUp(@PathVariable Integer productId){
        Result result = productService.startUp(productId);
        return result;
    }

    @PostMapping("/stopUp/{productId}")
    public Result<String> stopUp(@PathVariable Integer productId){
        Result result = productService.stopUp(productId);
        return result;
    }

    @PostMapping("/upload")
    public Result upload(@RequestParam("file")MultipartFile multipartFile){
        Result result = productService.upload(multipartFile);
        return result;
    }

    @PostMapping("/updateProduct")
    public Result<String> updateProduct(@RequestBody Product product){
        Result result = productService.updateProduct(product);
        return result;
    }

    @PostMapping("/updateUser")
    public Result<String> updateUser(@RequestBody User user){
        Result result = productService.updateUser(user);
        return result;
    }

    @PostMapping("/checkAdd")
    public Result<String> checkAdd(@RequestBody Check check){
        Result result = productService.checkAdd(check);
        return result;
    }

    @GetMapping("/checkList")
    public Result<List<Check>> checkList(){
        List<Check> checkList = productService.checkList();
        return Result.success(checkList);
    }

    @PostMapping("/updateUser1")
    public Result<String> updateUser1(@RequestBody User user){
        Result result = productService.updateUser1(user);
        return result;
    }
}
