package com.wzy.product.service.impl;
import cn.hutool.core.util.RandomUtil;
import com.wzy.common.domain.Check;
import com.wzy.common.domain.Product;
import com.wzy.common.domain.Rule;
import com.wzy.common.domain.User;
import com.wzy.common.result.Result;
import com.wzy.common.utils.FastUtil;
import com.wzy.common.utils.StringUtils;
import com.wzy.product.mapper.ProductMapper;
import com.wzy.product.service.ProductService;
import org.apache.commons.lang3.RandomUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;
import java.util.List;
import java.util.concurrent.TimeUnit;

@Service
public class ProductServiceImpl implements ProductService {

    @Autowired
    private ProductMapper productMapper;

    @Autowired
    private RedisTemplate<String,String> redisTemplate;

    @Autowired
    private FastUtil fastUtil;

    @Override
    public List<Product> findProductList() {
        return productMapper.findProductList();
    }

    @Override
    public List<Rule> ruleList() {
        return productMapper.ruleList();
    }

    @Override
    public Result ruleSave(Rule rule) {
        int code = RandomUtils.nextInt(10, 30);
        redisTemplate.opsForValue().set("code", String.valueOf(code),1,TimeUnit.MINUTES);
        rule.setStartNum(code);
        if (StringUtils.isEmpty(rule.getRuleName())) {
            return Result.error("规则名称不能为空!");
        }
        if (rule.getStartNum() > rule.getEndNum()) {
            return Result.error("截止次数不可小于起始次数!");
        }
        int i = productMapper.ruleSave(rule);
        if (i > 0) {
            return Result.success(i,"规则新增成功!");
        } else {
            return Result.error("请求失败");
        }
    }

    @Override
    public List<Product> choiceProduct() {
        return productMapper.choiceProduct();
    }

    @Override
    public Result productSave(Product product) {
        int i = productMapper.productSave(product);
        if (i > 0) {
            return Result.success(i,"新增成功");
        } else {
            return Result.error("请求失败");
        }
    }

    @Override
    public Result startUp(Integer productId) {
        int i = productMapper.startUp(productId);
        if (i > 0) {
            return Result.success(i,"开启砍价模式");
        } else {
            return Result.error("请求失败");
        }
    }

    @Override
    public Result stopUp(Integer productId) {
        int i = productMapper.stopUp(productId);
        if (i > 0) {
            return Result.success(i,"关闭砍价模式");
        } else {
            return Result.error("请求失败");
        }
    }

    @Override
    public Result upload(MultipartFile multipartFile) {
        String upload = null;
        try {
            upload = fastUtil.upload(multipartFile);
            if (null == upload) {
                return Result.error("上传失败");
            }
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
        return Result.success("http://165.154.161.166:8888/" + upload,"上传成功");
    }

    @Override
    public Result updateProduct(Product product) {
        double v = RandomUtils.nextDouble(0.1, 1);
        product.setBargainPrice(v);
        int i = productMapper.updateProduct(product);
        if (i > 0) {
            return Result.success(i,"砍价成功");
        } else {
            return Result.error("请求失败");
        }
    }

    @Override
    public Result updateUser(User user) {
        int i = productMapper.updateUser(user);
        if (i > 0) {
            return Result.success(i,"砍价次数更新成功");
        } else {
            return Result.error("请求失败");
        }
    }

    @Override
    public Result checkAdd(Check check) {
        double v = RandomUtils.nextDouble(0.1, 1);
        check.setBargainPrice(v);
        int i = productMapper.checkAdd(check);
        if (i > 0) {
            return Result.success(i,"砍价记录添加成功");
        } else {
            return Result.error("请求失败");
        }
    }

    @Override
    public List<Check> checkList() {
        return productMapper.checkList();
    }

    @Override
    public Result updateUser1(User user) {
        int i = productMapper.updateUser1(user);
        if (i > 0) {
            return Result.success(i,"砍价次数更新成功");
        } else {
            return Result.error("请求失败");
        }
    }
}
